@@ -2,54 +2,39 @@ package cmd
 
 import (
 	"fmt"
-	"io/ioutil"
 	"log"
 	"os"
-	"path/filepath"
 	"strings"
 	"text/tabwriter"
 
 	"github.com/beringresearch/macpine/host"
 	"github.com/beringresearch/macpine/qemu"
 	"github.com/spf13/cobra"
-	"gopkg.in/yaml.v3"
 )
 
 // listCmd lists Alpine instances
 var listCmd = &cobra.Command{
-	Use:   "list",
-	Short: "List instances.",
-	Run:   list,
+	Use:     "list",
+	Short:   "List instances.",
+	Run:     list,
+	Aliases: []string{"ls"},
 
 	DisableFlagsInUseLine: true,
 }
 
 func list(cmd *cobra.Command, args []string) {
-
-	userHomeDir, err := os.UserHomeDir()
-	if err != nil {
-		log.Fatal(err)
-	}
-
 	status := []string{}
-	config := []qemu.MachineConfig{}
+	configs := []qemu.MachineConfig{}
 	pid := []string{}
 
 	vmNames := host.ListVMNames()
-	for _, f := range vmNames {
-
-		machineConfig := qemu.MachineConfig{}
-		c, err := ioutil.ReadFile(filepath.Join(userHomeDir, ".macpine", f, "config.yaml"))
+	for _, vmName := range vmNames {
+		machineConfig, err := qemu.GetMachineConfig(vmName)
 		if err != nil {
 			log.Fatal(err)
 		}
 
-		err = yaml.Unmarshal(c, &machineConfig)
-		if err != nil {
-			log.Fatal(err)
-		}
-
-		config = append(config, machineConfig)
+		configs = append(configs, machineConfig)
 
 		s, p := host.Status(machineConfig)
 		status = append(status, s)
@@ -62,7 +47,7 @@ func list(cmd *cobra.Command, args []string) {
 
 	w := tabwriter.NewWriter(os.Stdout, 1, 1, 1, ' ', 0)
 	fmt.Fprintln(w, "NAME\tSTATUS\tSSH\tPORTS\tARCH\tPID\tTAGS\t")
-	for i, machine := range config {
+	for i, machine := range configs {
 		spacer := "    \t"
 		row := []string{
 			machine.Alias,
@@ -76,5 +61,4 @@ func list(cmd *cobra.Command, args []string) {
 		fmt.Fprintln(w, strings.Join(row, "    \t")+spacer)
 	}
 	w.Flush()
-
 }
