@@ -1,6 +1,7 @@
 package qemu
 
 import (
+	"bufio"
 	"errors"
 	"fmt"
 	"log"
@@ -12,6 +13,7 @@ import (
 	"strconv"
 	"strings"
 	"syscall"
+	"time"
 
 	"github.com/beringresearch/macpine/utils"
 	"golang.org/x/crypto/ssh"
@@ -28,7 +30,9 @@ type MachineConfig struct {
 	Memory       string   `yaml:"memory"`
 	Disk         string   `yaml:"disk"`
 	Mount        string   `yaml:"mount"`
+	MachineIP    string   `yaml:"machineip"`
 	Port         string   `yaml:"port"`
+	VMNet        bool     `yaml:"vmnet"`
 	SSHPort      string   `yaml:"sshport"`
 	SSHUser      string   `yaml:"sshuser"`
 	SSHPassword  string   `yaml:"sshpassword"`
@@ -43,9 +47,45 @@ func (c *MachineConfig) Exec(cmd string, root bool) error {
 	if cmd == "" {
 		return nil
 	}
-	host := "localhost:" + c.SSHPort
+	ip := c.MachineIP
+
+	if c.VMNet {
+		if ip == "localhost" || ip == "" {
+			log.Println("getting instance IP address from DHCP leases")
+
+			for {
+				ip = c.GetIPAddressByMac()
+
+				if ip != "" {
+					break
+				}
+				fmt.Print(".")
+				time.Sleep(4 * time.Second)
+			}
+
+			c.MachineIP = ip
+			config, err := yaml.Marshal(&c)
+
+			if err != nil {
+				c.Stop()
+				c.CleanPIDFile()
+				return err
+			}
+
+			err = os.WriteFile(filepath.Join(c.Location, "config.yaml"), config, 0644)
+			if err != nil {
+				c.Stop()
+				c.CleanPIDFile()
+				return err
+			}
+		}
+
+	}
+
+	host := ip + ":" + c.SSHPort
 	user := c.SSHUser
 	pwd := c.SSHPassword
+
 	if root && user != "root" {
 		user = "root"
 		if c.RootPassword == nil {
@@ -181,11 +221,15 @@ func (c *MachineConfig) Status() (string, int) {
 		pid, _ = c.GetInstancePID()
 
 		// check if stopped and return "Paused"
-		cmd := exec.Command("ps", "-o", "stat=", "-p", strconv.Itoa(pid))
-		out, err := cmd.Output()
-		if err != nil {
-			log.Fatalf("error checking status of qemu process: %v\n", err)
-		}
+		execArgs := []string{"-o", "stat=", "-p", strconv.Itoa(pid)}
+		execCmd := "ps"
+
+		cmd := exec.Command(execCmd, execArgs...)
+
+		out, _ := cmd.Output()
+		// if err != nil {
+		// 	log.Fatalf("error checking status of qemu process: %v\n", err)
+		// }
 		if strings.TrimSpace(string(out)) == "T" {
 			status = "Paused"
 		}
@@ -217,8 +261,8 @@ func (c *MachineConfig) Stop() error {
 			log.Println(c.Alias + " stopped")
 			return nil
 		} else {
-			pidFile := filepath.Join(c.Location, "alpine.pid")
-			return errors.New("error stopping, incorrect PID in " + pidFile + "?")
+			//pidFile := filepath.Join(c.Location, "alpine.pid")
+			return errors.New("failed to stop instance `" + c.Alias + "` due to inadequate preiveledges")
 		}
 	}
 	return nil
@@ -316,19 +360,26 @@ func (c *MachineConfig) GetAccel() string {
 // Start starts up an Alpine VM
 func (c *MachineConfig) Start() error {
 
-	exposedPorts := "user,id=net0,hostfwd=tcp::" + c.SSHPort + "-:22"
+	networkDevice := "user,id=net0,hostfwd=tcp::" + c.SSHPort + "-:22"
 
-	ports, err := utils.ParsePort(c.Port)
-	if err != nil {
-		log.Fatalf("Error configuring ports: %v\n", err)
+	if c.VMNet {
+		networkDevice = "vmnet-shared,id=net0"
 	}
-	for _, p := range ports {
-		hostp := strconv.Itoa(p.Host)
-		guestp := strconv.Itoa(p.Guest)
-		if p.Proto == utils.Tcp {
-			exposedPorts += ",hostfwd=tcp::" + hostp + "-:" + guestp
-		} else { // Udp
-			exposedPorts += ",hostfwd=udp::" + hostp + "-:" + guestp
+
+	// Only parse ports of using qemu's default slirp network
+	if !c.VMNet {
+		ports, err := utils.ParsePort(c.Port)
+		if err != nil {
+			log.Fatalf("Error configuring ports: %v\n", err)
+		}
+		for _, p := range ports {
+			hostp := strconv.Itoa(p.Host)
+			guestp := strconv.Itoa(p.Guest)
+			if p.Proto == utils.Tcp {
+				networkDevice += ",hostfwd=tcp::" + hostp + "-:" + guestp
+			} else { // Udp
+				networkDevice += ",hostfwd=udp::" + hostp + "-:" + guestp
+			}
 		}
 	}
 
@@ -394,8 +445,8 @@ func (c *MachineConfig) Start() error {
 		"-smp", "cpus=" + c.CPU + ",sockets=1,cores=" + c.CPU + ",threads=1",
 		"-drive", "if=virtio,file=" + filepath.Join(c.Location, c.Image),
 		"-nographic",
-		"-netdev", exposedPorts,
-		"-device", "e1000,netdev=net0,mac=" + c.MACAddress,
+		"-device", "virtio-net-pci,netdev=net0,mac=" + c.MACAddress,
+		"-netdev", networkDevice,
 		"-pidfile", filepath.Join(c.Location, "alpine.pid"),
 		"-chardev", "socket,id=char-serial,path=" + filepath.Join(c.Location,
 			"alpine.sock") + ",server=on,wait=off,logfile=" + filepath.Join(c.Location, "alpine.log"),
@@ -434,14 +485,6 @@ func (c *MachineConfig) Start() error {
 		return err
 	}
 
-	log.Println("awaiting ssh server...")
-	err = c.Exec("hwclock -s", true) // root=true i.e. run as root
-	if err != nil {
-		c.Stop()
-		c.CleanPIDFile()
-		return err
-	}
-
 	if c.Mount != "" {
 		basename := filepath.Base(c.Mount)
 		mntcmd := make([]string, 3)
@@ -464,6 +507,47 @@ func (c *MachineConfig) Start() error {
 	return nil
 }
 
+// AssignIP obtains machine IP address from bootpd.plist. Only applicale to machines created on
+// VMNet
+func (c *MachineConfig) GetIPAddressByMac() string {
+	ip := ""
+	mac := ""
+
+	file, err := os.Open("/var/db/dhcpd_leases")
+
+	if err != nil {
+		log.Fatalf("Error opening file: %v", err)
+	}
+
+	scanner := bufio.NewScanner(file)
+	for scanner.Scan() {
+
+		// Process each line in the dhcpd_leases file
+		line := scanner.Text()
+
+		// Check if the line contains an IP address
+		if strings.Contains(line, "ip_address") {
+			ip = strings.Split(strings.Fields(line)[0], "ip_address=")[1]
+		}
+
+		// Check if the line contains a MAC address
+		if strings.Contains(line, "hw_address") {
+			mac = strings.Split(strings.Fields(line)[0], "hw_address=1,")[1]
+		}
+
+		if mac == c.MACAddress {
+			return ip
+		}
+
+	}
+
+	// Check for any errors during scanning
+	if err := scanner.Err(); err != nil {
+		fmt.Println("error scanning /var/db/dhcpd_lease:", err)
+	}
+	return ""
+}
+
 // Launch macpine downloads a fresh image and creates a VM directory
 func (c *MachineConfig) Launch() error {
 
@@ -538,7 +622,39 @@ func (c *MachineConfig) Launch() error {
 
 	err = c.Start()
 	if err != nil {
-		return errors.New("unable launch a new machine. " + err.Error())
+		return errors.New("unable to launch a new machine. " + err.Error())
+	}
+
+	// Make sure DNS is set up correctly
+	err = c.Exec("echo 'nameserver 8.8.8.8' > /etc/resolv.conf", true)
+	if err != nil {
+		return errors.New("unable to set up DNS: " + err.Error())
+	}
+
+	err = c.Exec("apk update && apk add --no-cache dhclient", true)
+	if err != nil {
+		return errors.New("unable to install dhclient: " + err.Error())
+	}
+
+	err = c.Exec(`cat >/etc/dhcp/dhclient.conf <<EOL
+	option rfc3442-classless-static-routes code 121 = array of unsigned integer 8;
+
+	send host-name = gethostname();
+	request subnet-mask, broadcast-address, time-offset, routers,
+	        domain-name, domain-name-servers, domain-search, host-name,
+	        dhcp6.name-servers, dhcp6.domain-search, dhcp6.fqdn, dhcp6.sntp-servers,
+	        netbios-name-servers, netbios-scope, interface-mtu,
+	        rfc3442-classless-static-routes, ntp-servers;
+
+	prepend domain-name-servers 8.8.8.8, 8.8.4.4;
+	EOL`, true)
+	if err != nil {
+		return errors.New("unable to configure dhclient: " + err.Error())
+	}
+
+	err = c.Exec("rc-service networking restart", true)
+	if err != nil {
+		return errors.New("unable to restart networking services: " + err.Error())
 	}
 
 	// Resize disk on an alpine guest
@@ -549,8 +665,8 @@ func (c *MachineConfig) Launch() error {
 			return errors.New("unable to install dependencies: " + err.Error())
 		}
 
-		// send sfdisk command ,+ (<start>,<size>,<type>,<bootable>)
-		// default start (0), size + (all available), default type (linux data), default bootable (false)
+		//send sfdisk command ,+ (<start>,<size>,<type>,<bootable>)
+		//default start (0), size + (all available), default type (linux data), default bootable (false)
 		err = c.Exec(`echo ",+" | sfdisk --no-reread --partno 3 /dev/vda && partx -u /dev/vda`, true)
 		if err != nil {
 			return errors.New("error updating partition table: " + err.Error())
@@ -571,6 +687,62 @@ func (c *MachineConfig) Launch() error {
 	return nil
 }
 
+// CompressQemuDiskImage compresses the QEMU Disk image and overwrites it
+func (c *MachineConfig) CompressQemuDiskImage() error {
+	if !utils.CommandExists("qemu-img") {
+		return errors.New("qemu-img is not available on $PATH. ensure qemu is installed")
+	}
+
+	cmd := exec.Command("qemu-img", "convert", "-c", "-O", "qcow2", filepath.Join(c.Location, c.Image),
+		filepath.Join(c.Location, c.Image+"_compressed.qcow2"))
+
+	err := cmd.Run()
+	if err != nil {
+		return err
+	}
+
+	err = os.Remove(filepath.Join(c.Location, c.Image))
+	if err != nil {
+		return err
+	}
+
+	err = os.Rename(filepath.Join(c.Location, c.Image+"_compressed.qcow2"),
+		filepath.Join(c.Location, c.Image))
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
+// DecompressQemuDiskImage decompresses the QEMU Disk image and overwrites it
+func (c *MachineConfig) DecompressQemuDiskImage() error {
+	if !utils.CommandExists("qemu-img") {
+		return errors.New("qemu-img is not available on $PATH. ensure qemu is installed")
+	}
+
+	cmd := exec.Command("qemu-img", "convert", "-O", "qcow2", "-p", filepath.Join(c.Location, c.Image),
+		filepath.Join(c.Location, c.Image+"_decompressed.qcow2"))
+
+	err := cmd.Run()
+	if err != nil {
+		return err
+	}
+
+	err = os.Remove(filepath.Join(c.Location, c.Image))
+	if err != nil {
+		return err
+	}
+
+	err = os.Rename(filepath.Join(c.Location, c.Image+"_decompressed.qcow2"),
+		filepath.Join(c.Location, c.Image))
+	if err != nil {
+		return err
+	}
+
+	return nil
+}
+
 // ResizeQemuDiskImage resizes a qcow2 disk image
 func (c *MachineConfig) ResizeQemuDiskImage() error {
 	if !utils.CommandExists("qemu-img") {
@@ -621,6 +793,12 @@ func (c *MachineConfig) CleanPIDFile() {
 
 func (c *MachineConfig) GetInstancePID() (int, error) {
 	pidFile := filepath.Join(c.Location, "alpine.pid")
+
+	_, err := os.Open(pidFile)
+	if err != nil {
+		return 0, errors.New("you don't have enough priveledges to view " + c.Alias + ". run sudo alpine list")
+	}
+
 	vmPID, err := os.ReadFile(pidFile)
 	if err != nil {
 		return 0, err
